@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2010 IBM Corporation and others.
+ * Copyright (c) 2000, 2011 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -127,9 +127,21 @@ public Menu (Control parent) {
  * @see Widget#getStyle
  */
 public Menu (Decorations parent, int style) {
-	super (parent, checkStyle (style));
+	checkSubclass ();
+	checkParent (parent);
+	this.style = checkStyle(style);
+	if (parent != null) {
+		display = parent.display;
+	} else {
+		display = Display.getCurrent ();
+		if (display == null) display = Display.getDefault ();
+		if (!display.isValidThread ()) {
+			error (SWT.ERROR_THREAD_INVALID_ACCESS);
+		}
+	}
 	this.parent = parent;
-	createWidget ();
+	reskinWidget();
+	createWidget();
 }
 
 /**
@@ -188,6 +200,26 @@ public Menu (MenuItem parentItem) {
 	this (checkNull (parentItem).parent);
 }
 
+Menu (Display display) {
+	if (display == null) display = Display.getCurrent ();
+	if (display == null) display = Display.getDefault ();
+	if (!display.isValidThread ()) {
+		error (SWT.ERROR_THREAD_INVALID_ACCESS);
+	}
+	this.display = display;
+	this.style = SWT.BAR;
+	reskinWidget();
+	createWidget();
+}
+
+Menu (Display display, NSMenu nativeMenu) {
+	this.display = display;
+	this.style = SWT.DROP_DOWN;
+	this.nsMenu = nativeMenu;
+	reskinWidget();
+	createWidget();
+}
+
 static Control checkNull (Control control) {
 	if (control == null) SWT.error (SWT.ERROR_NULL_ARGUMENT);
 	return control;
@@ -203,6 +235,16 @@ static MenuItem checkNull (MenuItem item) {
 	return item;
 }
 
+void checkParent (Widget parent) {
+	// A null parent is okay when the app menu bar is in use.
+	if (parent == null && Display.getDefault().appMenuBar == null) error (SWT.ERROR_NULL_ARGUMENT);
+	if (parent != null) {
+		if (parent.isDisposed ()) error (SWT.ERROR_INVALID_ARGUMENT);
+		parent.checkWidget ();
+		parent.checkOpen ();
+	}
+}
+
 static int checkStyle (int style) {
 	return checkBits (style, SWT.POP_UP, SWT.BAR, SWT.DROP_DOWN, 0, 0, 0);
 }
@@ -294,30 +336,79 @@ public void addMenuListener (MenuListener listener) {
 
 void createHandle () {
 	display.addMenu (this);
-	NSMenu widget = (NSMenu)new SWTMenu().alloc();
-	widget = widget.initWithTitle(NSString.string());
-	widget.setAutoenablesItems(false);
-	widget.setDelegate(widget);	
-	nsMenu = widget;	
+	if (nsMenu == null) {
+		NSMenu widget = (NSMenu)new SWTMenu().alloc();
+		widget = widget.initWithTitle(NSString.string());
+		widget.setAutoenablesItems(false);
+		widget.setDelegate(widget);	
+		nsMenu = widget;	
+	} else {
+		nsMenu.retain();
+		long /*int*/ cls = OS.object_getClass(nsMenu.id);
+		long /*int*/ dynNSMenu_class = display.createMenuSubclass(cls, "SWTSystemMenu", true);
+		if (cls != dynNSMenu_class) {
+			OS.object_setClass(nsMenu.id, dynNSMenu_class);
+		}
+		nsMenu.setDelegate(nsMenu);	
+	}
 }
 
 void createItem (MenuItem item, int index) {
 	if (!(0 <= index && index <= itemCount)) error (SWT.ERROR_INVALID_RANGE);
-	NSMenuItem nsItem = null;
-	if ((item.style & SWT.SEPARATOR) != 0) {
-		nsItem = NSMenuItem.separatorItem();
-		nsItem.retain();
+	boolean add = true;
+	NSMenuItem nsItem = item.nsItem;
+	if (nsItem == null) {
+		if ((item.style & SWT.SEPARATOR) != 0) {
+			nsItem = NSMenuItem.separatorItem();
+			nsItem.retain();
+		} else {
+			nsItem = (NSMenuItem)new SWTMenuItem().alloc();
+			NSString empty = NSString.string();
+			nsItem.initWithTitle(empty, 0, empty);
+			nsItem.setTarget(nsItem);
+			nsItem.setAction(OS.sel_sendSelection);
+		}
+		item.nsItem = nsItem;
 	} else {
-		nsItem = (NSMenuItem)new SWTMenuItem().alloc();
-		NSString empty = NSString.string();
-		nsItem.initWithTitle(empty, 0, empty);
+		long /*int*/ cls = OS.object_getClass(nsItem.id);
+		long /*int*/ dynNSMenuItem_class = display.createMenuItemSubclass(cls, "SWTSystemMenuItem", true);
+		if (cls != dynNSMenuItem_class) {
+			OS.object_setClass(nsItem.id, dynNSMenuItem_class);
+		}
+		nsItem.retain();
+		item.nsItemAction = nsItem.action();
+		item.nsItemTarget = nsItem.target();
 		nsItem.setTarget(nsItem);
 		nsItem.setAction(OS.sel_sendSelection);
+
+		// Sync native item type to Item's style.
+		int type = SWT.PUSH;
+		if (nsItem.isSeparatorItem()) type = SWT.SEPARATOR;
+		if (nsItem.submenu() != null) type = SWT.CASCADE;
+		item.style |= type;
+		
+		// Sync native item text to Item's text.
+		item.text = nsItem.title().getString();
+		
+		// Sync native key equivalent to MenuItem's accelerator.
+		// The system menu on OS X only uses command and option, so it's
+		// safe to just check for those two key masks.
+		long /*int*/ keyMask = nsItem.keyEquivalentModifierMask();
+		NSString keyEquivString = nsItem.keyEquivalent();
+		long /*int*/ keyEquiv = 0;
+		if (keyEquivString != null) {
+			keyEquiv = keyEquivString.characterAtIndex(0);
+			if ((keyMask & OS.NSCommandKeyMask) != 0) keyEquiv |= SWT.COMMAND;
+			if ((keyMask & OS.NSAlternateKeyMask) != 0) keyEquiv |= SWT.ALT;
+			item.accelerator = (int) keyEquiv;
+		}
+		add = false;
 	}
-	item.nsItem = nsItem;
 	item.createJNIRef();
 	item.register();
-	nsMenu.insertItem(nsItem, index);
+	if (add) {
+		nsMenu.insertItem(nsItem, index);
+	}
 	if (itemCount == items.length) {
 		MenuItem [] newItems = new MenuItem [items.length + 4];
 		System.arraycopy (items, 0, newItems, 0, items.length);
@@ -325,17 +416,19 @@ void createItem (MenuItem item, int index) {
 	}
 	System.arraycopy (items, index, items, index + 1, itemCount++ - index);
 	items [index] = item;
-	NSMenu emptyMenu = item.createEmptyMenu ();
-	if (emptyMenu != null) {
-		nsItem.setSubmenu (emptyMenu);
-		emptyMenu.release();
-	}
-	if (display.menuBar == this) {
-		NSApplication application = display.application;
-		NSMenu menubar = application.mainMenu();
-		if (menubar != null) {
-			nsItem.setMenu(null);
-			menubar.insertItem(nsItem, index + 1);
+	if (add) {
+		NSMenu emptyMenu = item.createEmptyMenu ();
+		if (emptyMenu != null) {
+			nsItem.setSubmenu (emptyMenu);
+			emptyMenu.release();
+		}
+		if (display.menuBar == this) {
+			NSApplication application = display.application;
+			NSMenu menubar = application.mainMenu();
+			if (menubar != null) {
+				nsItem.setMenu(null);
+				menubar.insertItem(nsItem, index + 1);
+			}
 		}
 	}
 	//TODO - find a way to disable the menu instead of each item
@@ -500,6 +593,24 @@ String getNameText () {
 	return result;
 }
 
+/**
+ * Returns the orientation of the receiver, which will be one of the
+ * constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
+ *
+ * @return the orientation style
+ * 
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.7
+ */
+public int getOrientation () {
+	checkWidget ();
+	return style & (SWT.LEFT_TO_RIGHT | SWT.RIGHT_TO_LEFT);
+}
+
 /**
  * Returns the receiver's parent, which must be a <code>Decorations</code>.
  *
@@ -567,6 +678,7 @@ public Menu getParentMenu () {
  */
 public Shell getShell () {
 	checkWidget ();
+	if (this == display.appMenuBar) return null;
 	return parent.getShell ();
 }
 
@@ -590,7 +702,10 @@ public Shell getShell () {
 public boolean getVisible () {
 	checkWidget ();
 	if ((style & SWT.BAR) != 0) {
-		return this == parent.menuShell ().menuBar;
+		if (this == display.appMenuBar) 
+			return display.application.isActive();
+		else
+			return this == parent.menuShell ().menuBar;
 	}
 	if ((style & SWT.POP_UP) != 0) {
 		Menu [] popups = display.popups;
@@ -645,6 +760,7 @@ public int indexOf (MenuItem item) {
  */
 public boolean isEnabled () {
 	checkWidget ();
+	if (this == display.appMenuBar) return getEnabled();
 	Menu parentMenu = getParentMenu ();
 	if (parentMenu == null) {
 		return getEnabled () && parent.isEnabled ();
@@ -790,7 +906,7 @@ void releaseHandle () {
 void releaseParent () {
 	super.releaseParent ();
 	if (cascade != null) cascade.setMenu (null);
-	if ((style & SWT.BAR) != 0 && this == parent.menuBar) {
+	if ((style & SWT.BAR) != 0 && parent != null && this == parent.menuBar) {
 		parent.setMenuBar (null);
 	}
 }
@@ -973,6 +1089,24 @@ public void setLocation (Point location) {
 	setLocation (location.x, location.y);
 }
 
+/**
+ * Sets the orientation of the receiver, which must be one
+ * of the constants <code>SWT.LEFT_TO_RIGHT</code> or <code>SWT.RIGHT_TO_LEFT</code>.
+ * <p>
+ *
+ * @param orientation new orientation style
+ * 
+ * @exception SWTException <ul>
+ *    <li>ERROR_WIDGET_DISPOSED - if the receiver has been disposed</li>
+ *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the receiver</li>
+ * </ul>
+ * 
+ * @since 3.7  
+ */
+public void setOrientation (int orientation) {
+    checkWidget ();
+}
+
 /**
  * Marks the receiver as visible if the argument is <code>true</code>,
  * and marks it invisible otherwise. 
@@ -996,6 +1130,7 @@ public void setVisible (boolean visible) {
 		display.addPopup (this);
 	} else {
 		display.removePopup (this);
+		_setVisible(false);
 	}
 }
 	
