@@ -1,5 +1,5 @@
 /*******************************************************************************
- * Copyright (c) 2000, 2009 IBM Corporation and others.
+ * Copyright (c) 2000, 2011 IBM Corporation and others.
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
  * which accompanies this distribution, and is available at
@@ -104,6 +104,11 @@ boolean acceptsFirstResponder(long /*int*/ id, long /*int*/ sel) {
 	return super.acceptsFirstResponder (id, sel);
 }
 
+boolean accessibilityIsIgnored(long /*int*/ id, long /*int*/ sel) {
+	if (id == view.id) return true;
+	return super.accessibilityIsIgnored(id, sel);	
+}
+
 /**
  * Adds the listener to the collection of listeners who will
  * be notified when the receiver's text is modified, by sending
@@ -521,6 +526,12 @@ public int getTextLimit () {
     return textLimit;
 }
 
+boolean handleIsAccessible(long /*int*/ id) {
+	// All subviews of a Spinner can have their accessible properties overridden.
+	// The top-level NSView is already ignored, so we don't need to test for that.
+	return true;
+}
+
 boolean isEventView (long /*int*/ id) {
 	return true;
 }
